/*
 * Copyright 2011 Andres Gomez Casanova
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sf.mgaip;

import java.io.File;
import java.io.FilenameFilter;
import java.util.Locale;

import org.apache.maven.doxia.sink.Sink;
import org.apache.maven.doxia.siterenderer.Renderer;
import org.apache.maven.project.MavenProject;
import org.apache.maven.reporting.AbstractMavenReport;

/**
 * Inserts Google Ads in the generated Maven site.
 * <p>
 * <b>Control Version</b>
 * <p>
 * <ul>
 * <li>1.0 Class creation.</li>
 * </ul>
 *
 * @author Andres Gomez Casanova <a
 *         href="mailto:a n g o c a at y a h o o dot c o m">(AngocA)</a>
 * @version 1.0 20110507
 * @goal insert
 * @phase post-site
 * @since 1.0
 */
public final class AdInserterMojo extends AbstractMavenReport {

    /**
     * Custom script to insert in the vertical zone as a powered by.
     * <p>
     *
     * @parameter
     */
    String customPoweredBy;

    /**
     * Link to associate in the I like Facebook button.
     * <p>
     * The link has to be in this form:
     * <p>
     * <code>https%3A%2F%2Fwww.facebook.com%2Fpages%2FZemucan%2F138284808329
     * </code>
     * <p>
     * Replacing the ':' by %3A, '/' by %2F
     * <p>
     *
     * @parameter
     */
    String facebookLink;

    /**
     * Identifier of the Google Account. This is the google_ad_client field.
     *
     * @parameter default-value="ca-pub-9663595078690574"
     */
    String googleId;

    /**
     * Height of the horizontal ad.
     *
     * @parameter default-value=90
     */
    int horizontalAdHeight;
    /**
     * Identifier of the Horizontal Ad. This is the google_ad_slot field.
     * <p>
     * This is placed everywhere in the text.
     *
     * @parameter default-value="2519400815"
     */
    String horizontalAdId;
    /**
     * Width of the horizontal ad.
     *
     * @parameter default-value=728
     */
    int horizontalAdWidth;
    /**
     * Height of the logo ad.
     *
     * @parameter default-value=200
     */
    int logoAdHeight;
    /**
     * Identifier of the Logo Ad. This is the google_ad_slot field.
     * <p>
     * This could be used as a logo replacement.
     *
     * @parameter default-value="3789968358"
     */
    String logoAdId;
    /**
     * Width of the logo ad.
     *
     * @parameter default-value=200
     */
    int logoAdWidth;
    /**
     * Directory where reports will go.
     *
     * @parameter expression="${project.reporting.outputDirectory}"
     * @required
     * @readonly
     */
    private File outputDirectory;
    /**
     * @parameter default-value="${project}"
     * @required
     * @readonly
     */
    private MavenProject project;
    /**
     * @component
     * @required
     * @readonly
     */
    private Renderer siteRenderer;
    /**
     * Height of the unique ad.
     *
     * @parameter default-value=60
     */
    int uniqueAdHeight;
    /**
     * Identifier of the Unique Ad. This is the google_ad_slot field.
     * <p>
     * Placed inline the text.
     *
     * @parameter default-value="1788050126"
     */
    String uniqueAdId;
    /**
     * Width of the unique ad.
     *
     * @parameter default-value=234
     */
    int uniqueAdWidth;
    /**
     * Height of the vertical ad.
     *
     * @parameter default-value=600
     */
    int verticalAdHeight;
    /**
     * Identifier of the Vertical Ad. This is the google_ad_slot field.
     * <p>
     * This is placed at the vertical left side of the site, as a powered by
     * icon.
     *
     * @parameter default-value="9409358178"
     */
    String verticalAdId;
    /**
     * Width of the vertical ad.
     *
     * @parameter default-value=120
     */
    int verticalAdWidth;

    /**
     * Creates the report via Sink.
     */
    private void createReport() {
        // FIXME when re-executing the plugin, the report becomes corrupt.

        // Retrieves the Sink API.
        Sink sink = (Sink) getSink();

        // Header.
        sink.head();
        sink.title();
        sink.text("mgaip plugin configuration");
        sink.title_();
        sink.head_();

        // Body.
        // Google Ads
        sink.body();
        sink.section1();
        sink.sectionTitle1();
        sink.text("Google Ads");
        sink.sectionTitle1_();

        sink.paragraph();
        sink.text("These are the configured ads for user ");
        sink.bold();
        sink.text(this.googleId);
        sink.bold_();
        sink.paragraph_();

        sink.table();

        sink.tableRow();
        sink.tableHeaderCell();
        sink.text("Ad type");
        sink.tableHeaderCell_();
        sink.tableHeaderCell();
        sink.text("Slot");
        sink.tableHeaderCell_();
        sink.tableHeaderCell();
        sink.text("Width");
        sink.tableHeaderCell_();
        sink.tableHeaderCell();
        sink.text("Height");
        sink.tableHeaderCell_();
        sink.tableRow_();

        sink.tableRow();
        sink.tableCell();
        sink.text("Horizontal");
        sink.tableCell_();
        sink.tableCell();
        sink.text(this.horizontalAdId);
        sink.tableCell_();
        sink.tableCell();
        sink.text("" + this.horizontalAdWidth);
        sink.tableCell_();
        sink.tableCell();
        sink.text("" + this.horizontalAdHeight);
        sink.tableCell_();
        sink.tableRow_();

        sink.tableRow();
        sink.tableCell();
        sink.text("Logo");
        sink.tableCell_();
        sink.tableCell();
        sink.text(this.logoAdId);
        sink.tableCell_();
        sink.tableCell();
        sink.text("" + this.logoAdWidth);
        sink.tableCell_();
        sink.tableCell();
        sink.text("" + this.logoAdHeight);
        sink.tableCell_();
        sink.tableRow_();

        sink.tableRow();
        sink.tableCell();
        sink.text("Unique");
        sink.tableCell_();
        sink.tableCell();
        sink.text(this.uniqueAdId);
        sink.tableCell_();
        sink.tableCell();
        sink.text("" + this.uniqueAdWidth);
        sink.tableCell_();
        sink.tableCell();
        sink.text("" + this.uniqueAdHeight);
        sink.tableCell_();
        sink.tableRow_();

        sink.tableRow();
        sink.tableCell();
        sink.text("Vertical");
        sink.tableCell_();
        sink.tableCell();
        sink.text(this.verticalAdId);
        sink.tableCell_();
        sink.tableCell();
        sink.text("" + this.verticalAdWidth);
        sink.tableCell_();
        sink.tableCell();
        sink.text("" + this.verticalAdHeight);
        sink.tableCell_();
        sink.tableRow_();

        sink.table_();

        sink.section1_();

        // Facebook I like.
        if (this.facebookLink != null && !this.facebookLink.equals("")) {
            sink.section1();
            sink.sectionTitle1();
            sink.text("Facebook I Like");
            sink.sectionTitle1_();

            sink.paragraph();
            sink.text("This is the configuration for the Facebook I Like "
                    + "buttom.");
            sink.lineBreak();
            sink.text("Link: ");
            sink.bold();
            sink.link(this.facebookLink);
            sink.text(this.facebookLink);
            sink.link_();
            sink.bold_();
            sink.paragraph_();
            sink.section1_();
        }

        // Custom powered by.
        if (this.customPoweredBy != null && !this.customPoweredBy.equals("")) {
            sink.section1();
            sink.sectionTitle1();
            sink.text("Custom Powered By");
            sink.sectionTitle1_();

            sink.paragraph();
            sink.text("This is the custom configuration:");
            sink.paragraph_();

            sink.paragraph();
            sink.monospaced();
            sink.rawText(this.customPoweredBy);
            sink.monospaced_();
            sink.paragraph_();
            sink.section1_();
        }

        sink.body_();

        sink.flush();
        sink.close();
    }

    /*
     * (non-Javadoc)
     *
     * @see org.apache.maven.plugin.Mojo#execute()
     */
    public void executeReport(final Locale/* ! */locale) {
        // Custom logger for debug messages.
        // this.setLog(new MgaipLogger());

        this.getLog().info("Starting mgaip");
        this.getLog().info("Google Id: " + this.googleId);
        this.getLog().debug("Vertical Ad: " + this.verticalAdId);
        this.getLog().debug("Horizontal Ad: " + this.horizontalAdId);
        this.getLog().debug("Unique Ad: " + this.uniqueAdId);
        this.getLog().debug("Logo Ad: " + this.logoAdId);

        this.createReport();

        this.replaceFiles();
    }

    /*
     * (non-Javadoc)
     *
     * @see
     * org.apache.maven.reporting.MavenReport#getDescription(java.util.Locale)
     */
    public String getDescription(final Locale/* ! */locale) {
        return "Includes Google publicity in the generated site.";
    }

    /*
     * (non-Javadoc)
     *
     * @see org.apache.maven.reporting.MavenReport#getName(java.util.Locale)
     */
    public String/* ! */getName(final Locale/* ! */locale) {
        return "Maven GoogleAds inserter plugin";
    }

    /*
     * (non-Javadoc)
     *
     * @see org.apache.maven.reporting.AbstractMavenReport#getOutputDirectory()
     */
    protected String/* ! */getOutputDirectory() {
        return outputDirectory.getAbsolutePath();
    }

    /*
     * (non-Javadoc)
     *
     * @see org.apache.maven.reporting.MavenReport#getOutputName()
     */
    public String/* ! */getOutputName() {
        return "mgaip";
    }

    /*
     * (non-Javadoc)
     *
     * @see org.apache.maven.reporting.AbstractMavenReport#getProject()
     */
    protected MavenProject/* ! */getProject() {
        return this.project;
    }

    /*
     * (non-Javadoc)
     *
     * @see org.apache.maven.reporting.AbstractMavenReport#getSiteRenderer()
     */
    protected Renderer/* ! */getSiteRenderer() {
        return this.siteRenderer;
    }

    /**
     * Writes in the log in the given level.
     *
     * @param level
     *            Level of the log to write: 1 debug, 2 info, 3 warn, 4 error.
     * @param message
     *            Message to print.
     */
    void log(final int level, final String/* ! */message) {
        switch (level) {
        case 1:
            this.getLog().debug(message);
            break;
        case 2:
            this.getLog().info(message);
            break;
        case 3:
            this.getLog().warn(message);
            break;
        case 4:
            this.getLog().error(message);
            break;
        default:
            this.getLog().info(message);
            break;
        }
    }

    /**
     * Prints the content of an exception.
     *
     * @param exception
     *            Exception to print.
     */
    void log(final Throwable/* ! */exception) {
        this.getLog().error(exception);
    }

    /**
     * Creates a list of files to modify and then replace the tags of them.
     */
    private void replaceFiles() {
        // Retrieves only the HTML files.
        final File file = this.outputDirectory;
        String[] files = file.list(new FilenameFilter() {

            /*
             * (non-Javadoc)
             *
             * @see java.io.FilenameFilter#accept(java.io.File,
             * java.lang.String)
             */
            public boolean accept(File dir, String name) {
                boolean ret = false;
                if (name.startsWith("mgaip")) {
                    ret = false;
                } else if (name.endsWith("html")) {
                    ret = true;
                }
                return ret;
            }

        }

        );

        Replacer replacer = new Replacer(this);

        if (files != null) {
            this.getLog().debug("Files to process:");
            for (int i = 0; i < files.length; i++) {
                final String filename = files[i];
                this.getLog().debug("File: " + filename);
                replacer.replace(this.outputDirectory.getAbsolutePath(),
                        filename);
            }
        } else {
            this.getLog().warn("No files to process.");
        }
    }
}
